#include <AT91SAM7X256.H>
#include <lib_at91sam7x256.h> 
#include <stdio.h>
#include <pio.h>

#include "LCD_N96/TFTLCD.h"
#include "LCD_N96/LCD_CLINE.h"
#include "tdelay/tdelay.h"
#include "spi/spi.h"

#define BORD_CLK	92160000

#define CSN			PA8
#define CE			PA10

#define CSN_HIGH		SET_PORTA=PA8;
#define CSN_LOW			RST_PORTA=PA8;

#define CE_HIGH			SET_PORTA=PA10;
#define CE_LOW			RST_PORTA=PA10;

unsigned char received_data[32];

#define TX_ADDR_WIDTH    5   
#define TX_PAYLOAD_WIDTH  32  

unsigned char TX_ADDRESS[TX_ADDR_WIDTH]  = {0xC1, 0x21, 0x12, 0x21, 0xC1};

/*************************************************************************/
//////////////////////// nRF24L01 SPI Commands/////////////////////////////

#define R_REGISTER	    0x00  
#define W_REGISTER		0x20  
#define R_RX_PAYLOAD    0x61 
#define W_TX_PAYLOAD    0xA0  

/*************************************************************************/
/////////////////// nRF24L01 Registers Address ////////////////////////////

#define CONFIG          0x00  
#define EN_AA           0x01  
#define EN_RXADDR       0x02 
#define SETUP_AW        0x03  
#define SETUP_RETR      0x04  
#define RF_CH           0x05  
#define RF_SETUP        0x06  
#define STATUS          0x07  
#define OBSERVE_TX      0x08  
#define CD              0x09  
#define RX_ADDR_P0      0x0A  
#define RX_ADDR_P1      0x0B  
#define RX_ADDR_P2      0x0C  
#define RX_ADDR_P3      0x0D  
#define RX_ADDR_P4      0x0E  
#define RX_ADDR_P5      0x0F  
#define TX_ADDR         0x10  
#define RX_PW_P0        0x11  
#define RX_PW_P1        0x12  
#define RX_PW_P2        0x13  
#define RX_PW_P3        0x14  
#define RX_PW_P4        0x15  
#define RX_PW_P5        0x16  
#define FIFO_STATUS     0x17  

unsigned char nRF24L01_Read_Register(unsigned char reg)
{
	unsigned char data;

	CSN_LOW;
	delay_ms(1);

    SPI_Write(AT91C_BASE_SPI0,0,reg);
	SPI_Write(AT91C_BASE_SPI0,0,0);	 
    SPI_Read(AT91C_BASE_SPI0);
	data = SPI_Read(AT91C_BASE_SPI0);

	CSN_HIGH;
	
    return data;
}

void nRF24L01_Write_Register(unsigned char reg, unsigned char value)
{
	CSN_LOW;
	delay_ms(1);
	
    SPI_Write(AT91C_BASE_SPI0,0,reg|W_REGISTER);
	delay_ms(1);
	SPI_Write(AT91C_BASE_SPI0,0,value);
	delay_ms(1);
	SPI_Read(AT91C_BASE_SPI0);
	delay_ms(1);

	CSN_HIGH;
}


void nRF24L01_Write_Buffer(unsigned char reg, unsigned char *pBuf, unsigned char bytes)
{
	unsigned char byte_ctr;

	CSN_LOW;
	delay_ms(1);

	SPI_Write(AT91C_BASE_SPI0,0,reg); 
	delay_ms(1);
	SPI_Read(AT91C_BASE_SPI0);
	delay_ms(1);

  	for(byte_ctr=0; byte_ctr<bytes; byte_ctr++)
	{
   		SPI_Write(AT91C_BASE_SPI0,0,*pBuf++);
		delay_ms(1);
		SPI_Read(AT91C_BASE_SPI0);           
		delay_ms(1);
	}

	CSN_HIGH;
}

void nRF24L01_Read_Buffer(unsigned char i)
{
	unsigned char j,bytes = TX_PAYLOAD_WIDTH;

	CSN_LOW;
	delay_ms(1);
	
  	SPI_Write(AT91C_BASE_SPI0,0,R_RX_PAYLOAD);
	delay_ms(1);  
	SPI_Read(AT91C_BASE_SPI0);
	delay_ms(1);
	i = i-1;

	for(j=i;j<bytes;j++)
	{
    	SPI_Write(AT91C_BASE_SPI0,0,0);
		delay_ms(1);
		received_data[j] = SPI_Read(AT91C_BASE_SPI0);
		delay_ms(1);
	}

	CSN_HIGH;
}

// Receive 64 Bytes
void Receive_Data(void)
{
	nRF24L01_Read_Buffer(1);
}

void RX_MODE(void)
{
	CE_LOW;
	delay_ms(1);

	nRF24L01_Write_Register(CONFIG, 0x0B);
	nRF24L01_Write_Register(EN_AA, 0x01);
	nRF24L01_Write_Register(EN_RXADDR, 0x01);
	nRF24L01_Write_Register(SETUP_AW, 0x03);
	nRF24L01_Write_Buffer(RX_ADDR_P0|W_REGISTER, TX_ADDRESS, TX_ADDR_WIDTH);
  	nRF24L01_Write_Register(RF_CH, 23);// F0= 2400 + RF_CH [MHz], F0 = 2423 [MHz]
	nRF24L01_Write_Register(RF_SETUP, 0x07); 
  	nRF24L01_Write_Register(RX_PW_P0, TX_PAYLOAD_WIDTH);
  	
	CE_HIGH;
	delay_ms(1);

}

int main(void) 
{
	unsigned char i;

	PORTA_OUTPUT = CE;
	CE_LOW;
	
	PORTA_OUTPUT = CSN;
	CSN_HIGH;

	SPI_Configure_PIO_MAKS(SPI0_PORT,SPI0_PIO_MASK);	
	SPI_Configure(AT91C_BASE_SPI0, AT91C_ID_SPI0, SPI_MASTER|SPI_ONE_DEVIDE|SPI_MOD_FDIS);	
	SPI_ConfigureNPCS(AT91C_BASE_SPI0, 0, SPI_MODE_0|SPI_8_BIT|SPI_SCBR(2000000,BORD_CLK)); 
	SPI_Enable(AT91C_BASE_SPI0);
	
	init_delay();	
	tftlcd_init();
	tftlcd_Fill(BLACK);

	delay_ms(10);
	RX_MODE();
	
	while(1)
	{
		Receive_Data();
		if(received_data[0] == 0xAA)
		{
			for(i=0;i<32;i++)
			{
				lprintf("%d - ",received_data[i]);
			}
			delay_ms(5000);
			tftlcd_clear();
			tftlcd_Fill(BLACK);
			tftlcd_gotoxy(0,0);
		}
	}

	//while(1);
}
